<?php
/**
 * @package JoomlaStats
 * @copyright Copyright (C) 2004-2009 JoomlaStats Team. All rights reserved.
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */



if( !defined( '_JS_STAND_ALONE' ) && !defined( '_JEXEC' ) ) {
	die( 'JS: No Direct Access' );
}





/**
 *  This class contain API (application programming interface) to JoomlaStats specially for modules.
 *
 *  Eg. of including JoomlaStats API
 * 	  require_once( dirname(__FILE__) .DIRECTORY_SEPARATOR. 'joomla' .DIRECTORY_SEPARATOR. 'administrator' .DIRECTORY_SEPARATOR. 'components' .DIRECTORY_SEPARATOR. 'com_joomlastats' .DIRECTORY_SEPARATOR. 'api' .DIRECTORY_SEPARATOR. 'module.php' );
 *
 *
 *  All methods are static
 */
class js_JSApiModule
{


	/** 
	 *  This function convert time period string from *.xml to SQL WHERE condition (in optimized way)
	 *
	 *
	 *  Below code for *.xml file:
	 *
        <param name="list_time_period"        type="list"     default="total"              label="List time period"         description="Choose time period to show country statistics for.&lt;br/>&lt;br/>eg.&lt;br/>&lt;b>For last week&lt;/b> - shows statistics for last week. If You have approximately constant number of visitors, value will be more or less constant.&lt;br/>&lt;b>This week&lt;/b> - shows statistics for current week. Monday is first day of week. If You have approximately constant number of visitors, value for 'This week' time period will change from 0 up to 'For last week'.">
            <option value="for_last_d-h-m_0-0-5">For last 5 minutes</option>
            <option value="for_last_d-h-m_0-0-10">For last 10 minutes</option>
            <option value="for_last_d-h-m_0-0-15">For last 15 minutes</option>
            <option value="for_last_d-h-m_0-0-20">For last 20 minutes</option>
            <option value="for_last_d-h-m_0-0-30">For last 30 minutes</option>
            <option value="for_last_d-h-m_0-1-0">For last hour</option>
            <option value="for_last_d-h-m_0-2-0">For last 2 hours</option>
            <option value="for_last_d-h-m_0-6-0">For last 6 hours</option>
            <option value="for_last_d-h-m_0-12-0">For last 12 hours</option>
            <option value="today">Today</option>
            <option value="this_week">This week</option>
            <option value="for_last_d-h-m_7-0-0">For last week</option>
            <option value="this_month">This month</option>
            <option value="for_last_d-h-m_31-0-0">For last month</option>
            <option value="for_last_d-h-m_92-0-0">For last 3 months</option>
            <option value="for_last_d-h-m_183-0-0">For last 6 months</option>
            <option value="this_year">This year</option>
            <option value="for_last_d-h-m_365-0-0">For last year</option>
            <option value="total">Total</option>
        </param>
	 *
	 *
	 *  NOTICE:
	 *    Function is complicated because SQL query is optimized!!
	 *
	 */
	function getConditionStringFromXmlTimePeriodList( $list_time_period ) {
		global $mainframe;

		if ($list_time_period == 'total')
			return '1=1';//total

		$sql_constr_time = '1=1';//total and wrong value


		if (substr($list_time_period, 0, 15) == 'for_last_d-h-m_') {
			$xml_day_minute_str = substr($list_time_period, 15);
			$day_minute_arr = explode('-', $xml_day_minute_str);
			$nbr_of_days = (int)$day_minute_arr[0];
			$nbr_of_hours = (int)$day_minute_arr[1];
			$nbr_of_minutes = (int)$day_minute_arr[2];

			if ($nbr_of_days == 0) 
				$sql_constr_time = 'v.visit_time >= DATE_SUB(NOW(), INTERVAL \''.$nbr_of_hours.':'.$nbr_of_minutes.'\' HOUR_MINUTE)';
			else if ( ($nbr_of_hours == 0) && ($nbr_of_minutes == 0) )
				$sql_constr_time = 'v.visit_date >= DATE_SUB(NOW(), INTERVAL \''.$nbr_of_days.'\' DAY)';
			else
				$sql_constr_time = 'CAST(CONCAT(v.visit_date, \' \', v.visit_time) AS DATETIME) >= DATE_SUB(NOW(), INTERVAL \''.$nbr_of_days.' '.$nbr_of_hours.':'.$nbr_of_minutes.'\' DAY_MINUTE)';
		} else {
			$offset = $mainframe->getCfg( 'offset' );
			$now_timestamp = time() + $offset*3600;
			//$now_date_str = gmdate('Y-m-d', $now_timestamp);
			//$now_time_str = gmdate('H:i:s', $now_timestamp);

			switch ($list_time_period)
			{
				case 'today':
					$today_date_str = gmdate('Y-m-d', $now_timestamp);
					$sql_constr_time = 'v.visit_date = \''.$today_date_str.'\'';
					break;
				case 'this_week':
					$year_date_str = gmdate('Y', $now_timestamp);
					$today_date_str = gmdate('Y-m-d', $now_timestamp);
					$sql_constr_time = '( YEAR(v.visit_date) = \''.$year_date_str.'\' AND WEEK(v.visit_date,3) = WEEK(\''.$today_date_str.'\',3) )';
					break;
				case 'this_month':
					$year_date_str = gmdate('Y', $now_timestamp);
					$month_date_str = gmdate('m', $now_timestamp);
					$sql_constr_time = '( YEAR(v.visit_date) = \''.$year_date_str.'\' AND MONTH(v.visit_date) = \''.$month_date_str.'\' )';
					break;
				case 'this_year':
					$year_date_str = gmdate('Y', $now_timestamp);
					$sql_constr_time = 'YEAR(v.visit_date) = \''.$year_date_str.'\'';
					break;
				default: //wrong value
					$sql_constr_time = '1=1';//total
					break;
			}
		}

		return $sql_constr_time;
	}



}

